require 'fox'
require 'tilt'
require 'sass'

# A builder process that compiles CSS from SASS.

# Cases
# 1. Success: built
# 2. Fail: threw an exception
# 3. Abort: dependencies aren't there
def build_css(request)
  # get source from target
  source = "src/sass/" + request[:target].gsub(/\.css$/, '.sass')

  # find out if theres dependencies
  t = Tilt.new(source)
  #Not sure if this is the right way to reach in and get the SASS engine...my Rubby-fu is weak.
  dependencies = t.instance_variable_get(:@engine).dependencies.map{|x| x.options[:filename]}

  # find out if all dependencies are met
  if Fox.ensure(request[:socket], dependencies)
    # Build it
    begin
      res = t.render nil

      # Success
      return {
        :exit   => :succeed,
        # optional
        :result => res,
        :log    => "Successfully built #{request[:target]}",
        :tags   => ["web"]
      }
    rescue
      # Failure
      return {
        :exit   => :fail,
        # optional
        :log    => "Failed to build #{request[:target]}",
        :tags   => ["web"]
      }
    end
  else
    # Abort
    return {
      :exit   => :abort,
      # optional
      :log    => "Dependencies not met for #{request[:target]}",
      :tags   => ["web"]
    }
  end
end

# Example of registering a handler for a target
Fox.register "default.css" { |req| build_css(req) }

Fox.register "target name" do |req|

  ##Your build code

  # general case: return a map:
  #
  # {
  #   :exit   => :succeed,
  #   :result => "the built target...",
  #   # optional
  #   :log    => "Dependencies not met for #{request[:target]}",
  #   :tags   => ["web"]
  # }

  # specific case: return just a successful result or nil.
  #
  # "my result"

  # specific case: succeed with log message
  #
  # "my result", "log..."

  # If your code needs to ensure that dependencies exist before continuing, use this
  #
  #     Fox.ensure socket, deps
  #
  # (It will throw an exception that's caught by the rubbygem, which will return an abort map on your behalf)

  # If your build fails for some other reason, feel free to raise hell:
  #
  #     raise "hell!"
  #
  # the rubbygem will catch and return a build failure map for you.
  
end
